const { merge } = require('webpack-merge');
const ESLintPlugin = require('eslint-webpack-plugin');
const LoadablePlugin = require('@loadable/webpack-plugin');
const { getProjectPath } = require('../common/utils');
const baseConfig = require('./webpack.base');

module.exports = function (appConfig, extendEnv, { isServer, isHotServer, isProd }) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer,
      isHotServer,
      isProd
    }),
    {
      // 入口文件
      entry: getProjectPath('/src/client-entry.tsx'),
      resolve: {
        fallback: {
          fs: false
        }
      },
      // 输出信息
      output: {
        // 输出文件路径
        path: getProjectPath('/devPublic'),
        publicPath: '/',
        filename: `${appConfig.baseStaticJsPath}/[name].js`,
        chunkFilename: `${appConfig.baseStaticJsPath}/[name].js`
      },
      plugins: [
        // eslint只在本地开发时检测
        new ESLintPlugin({
          exclude: 'node_modules',
          threads: 4,
          failOnError: false,
          lintDirtyModulesOnly: isProd,
          extensions: ['tsx', 'ts', 'js']
        }),
        new LoadablePlugin()
      ]
    }
  );
};
