const { merge } = require('webpack-merge');
const ESLintPlugin = require('eslint-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ip = require('ip');
const FriendlyErrorsPlugin = require('@soda/friendly-errors-webpack-plugin');
const { getProjectPath } = require('../common/utils');
const baseConfig = require('./webpack.base');

const HOST = process.env.HOST || ip.address() || 'localhost';

module.exports = function (appConfig, extendEnv, { isServer, isHotServer, isProd }) {
  return merge(
    baseConfig(appConfig, extendEnv, {
      isServer,
      isHotServer,
      isProd
    }),
    {
      entry: getProjectPath('/src/index.tsx'),
      output: {
        publicPath: '/',
        path: getProjectPath('/dist'),
        pathinfo: true,
        filename: '[name].js',
        chunkFilename: '[name].chunk.js'
      },
      plugins: [
        // eslint只在本地开发时检测
        new ESLintPlugin({
          exclude: 'node_modules',
          threads: 4,
          failOnError: false,
          lintDirtyModulesOnly: isProd,
          extensions: ['tsx', 'ts', 'js']
        }),
        // 相关配置文档：https://github.com/geowarin/friendly-errors-webpack-plugin
        new FriendlyErrorsPlugin({
          compilationSuccessInfo: {
            messages: [
              `localhost运行链接: http://localhost:${appConfig.port}${appConfig.startOpenPath}\n`,
              `ip运行链接: http://${HOST}:${appConfig.port}${appConfig.startOpenPath}\n`
            ]
          },
          onErrors: null,
          clearConsole: true
        }),
        new HtmlWebpackPlugin({
          inject: true,
          template: getProjectPath(appConfig.htmlPluginTemplate),
          favicon: appConfig.htmlPluginFavicon ? getProjectPath(appConfig.htmlPluginFavicon) : undefined
        })
      ],
      resolve: {
        fallback: {
          fs: false
        }
      },
      optimization: {
        minimize: false,
        splitChunks: {
          chunks: 'all',
          name: false
        },
        runtimeChunk: true
      }
    }
  );
};
